#!/usr/local/bin/perl
# index.cgi
# List all LDAP users for editing

require './ldap-useradmin-lib.pl';
&ui_print_header(undef, $module_info{'desc'}, "", "intro", 1, 1);

# Make sure the LDAP NSS client config file exists, or the needed information
# has been provided
if ($config{'auth_ldap'}) {
	if (!-r $config{'auth_ldap'}) {
		print "<p>",&text('index_econfig',
		    "<tt>$config{'auth_ldap'}</tt>",
		    "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	$nss = &get_nss_config();
	if ($nss->{'pidfile'} || $nss->{'directory'}) {
		print "<p>",&text('index_econfig2',
		    "<tt>$config{'auth_ldap'}</tt>",
		    "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	}
else {
	if (!$config{'ldap_host'} || !$config{'login'} || !$config{'pass'} ||
	    !$config{'user_base'} || !$config{'group_base'}) {
		print "<p>",&text('index_ehost',
			"$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	}

# Make sure the LDAP Perl module is installed, and if not offer to install
if (!$got_net_ldap) {
	local @needs;
	foreach $m ("Convert::ASN1", "Net::LDAP") {
		eval "use $m";
		push(@needs, $m) if ($@);
		}
	$missing = &urlize(join(" ", @needs));
	print "<p>",&text('index_eperl', "<tt>$missing</tt>",
			  "/cpan/download.cgi?source=3&cpan=$missing&mode=2&".
			  "return=/$module_name/&returndesc=".
			  &urlize($text{'index_return'})),"<p>\n";
	print "$text{'index_eperl2'}\n";
	print "<pre>$net_ldap_error</pre>\n";
	&ui_print_footer("/", $text{'index'});
	exit;
	}

# Try to login .. may fail
$ldap = &ldap_connect(1);
if (!ref($ldap)) {
	print "<p>",&text('index_eldap', $ldap,
		  "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
	&ui_print_footer("/", $text{'index'});
	exit;
	}

if ($config{'imap_host'}) {
	# Make sure the IMAP Perl module is installed, and if not offer
	# to install
	if (!$got_net_imap) {
		print "<p>",&text('index_eperl', "<tt>Net::IMAP</tt>",
			  "/cpan/download.cgi?source=3&cpan=Net::IMAP&mode=2&".
			  "return=/$module_name/&returndesc=".
			  &urlize($text{'index_return'})),"<p>\n";
		print "$text{'index_eperl2'}\n";
		print "<pre>$net_imap_error</pre>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}

	# Try to connect to the IMAP server
	$imap = &imap_connect(1);
	if (!ref($imap)) {
		print "<p>",&text('index_eimap', $imap,
		  "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	}

# If using MD5, make sure needed perl modules or programs are installed
if ($config{'md5'} == 1) {
	# Check if MD5 perl module is installed, and offer to install
	&foreign_require("useradmin", "user-lib.pl");
	if ($err = &useradmin::check_md5()) {
		print "<p>",&text('index_emd5',
				  "../config.cgi?$module_name",
				  "<tt>$err</tt>",
				  "../cpan/download.cgi?source=3&cpan=Digest::MD5&mode=2&return=/$module_name/&returndesc=".&urlize($text{'index_return'})),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	}
elsif ($config{'md5'} == 3) {
	# Check if slappasswd is installed
	if (!&has_command($config{'slappasswd'})) {
		print "<p>",&text('index_eslappasswd',
				  "../config.cgi?$module_name",
				  "<tt>$config{'slappasswd'}</tt>"),"<p>\n";
		&ui_print_footer("/", $text{'index'});
		exit;
		}
	}

# Count the number of users and groups
$base = &get_user_base();
$rv = $ldap->search(base => $base,
		    filter => '(objectClass=posixAccount)');
$ucount = $rv->count;
$base = &get_group_base();
$rv = $ldap->search(base => $base,
		    filter => '(objectClass=posixGroup)');
$gcount = $rv->count;

# Get the list of users and groups
if ($ucount <= $mconfig{'display_max'}) {
	@allulist = &list_users();
	@ulist = &useradmin::list_allowed_users(\%access, \@allulist);
	}
if ($gcount <= $mconfig{'display_max'}) {
	@allglist = &list_groups();
	@glist = &useradmin::list_allowed_groups(\%access, \@allglist);
	}

$form = 0;
if ($ucount > $mconfig{'display_max'}) {
	# Show user search form
	print "<b>$text{'index_toomany'}</b><br>\n";
	print "<form action=search_user.cgi>\n";
	print "<b>$text{'index_find'}</b> <select name=field>\n";
	print "<option value=uid selected>$text{'user'}\n";
	print "<option value=cn>$text{'real'}\n";
	print "<option value=loginShell>$text{'shell'}\n";
	print "<option value=homeDirectory>$text{'home'}\n";
	print "<option value=uidNumber>$text{'uid'}\n";
	print "</select> <select name=match>\n";
	print "<option value=0 checked>$text{'index_equals'}\n";
	print "<option value=1>$text{'index_contains'}\n";
	print "<option value=2>$text{'index_nequals'}\n";
	print "<option value=3>$text{'index_ncontains'}\n";
	print "<option value=6>$text{'index_lower'}\n";
	print "<option value=7>$text{'index_higher'}\n";
	print "</select> <input name=what size=15>&nbsp;&nbsp;\n";
	print "<input type=submit value=\"$text{'find'}\"></form>\n";
	}
elsif (@ulist) {
	# Show table of all users
	if ($access{'ucreate'}) {
		print "<a href='edit_user.cgi?new=1'>$text{'index_uadd'}</a>\n";
		print "&nbsp;&nbsp;\n";
		}
	if ($access{'batch'}) {
		print "<a href=\"batch_form.cgi\">$text{'index_batch'}</a>\n";
		}
	print "<br>\n";
	@ulist = &useradmin::sort_users(\@ulist, $mconfig{'sort_mode'});
	&useradmin::users_table(\@ulist, $form++, 1, 0);
	}
else {
	# No users
	$base = &get_user_base();
	print "<b>",&text('index_unone', "<tt>$base</tt>"),"</b><p>\n";
	}
if ($access{'ucreate'}) {
	print "<a href='edit_user.cgi?new=1'>$text{'index_uadd'}</a>\n";
	print "&nbsp;&nbsp;\n";
	}
if ($access{'batch'}) {
	print "<a href=\"batch_form.cgi\">$text{'index_batch'}</a>\n";
	}
print "<br>\n";

# Get the list of groups
if ($gcount > $mconfig{'display_max'}) {
	# Show group search form
	print "<hr>\n";
	print "<b>$text{'index_gtoomany'}</b><br>\n";
	print "<form action=search_group.cgi>\n";
	print "<b>$text{'index_gfind'}</b> <select name=field>\n";
	print "<option value=cn selected>$text{'gedit_group'}\n";
	print "<option value=memberUid>$text{'gedit_members'}\n";
	print "<option value=gidNumber>$text{'gedit_gid'}\n";
	print "</select> <select name=match>\n";
	print "<option value=0 checked>$text{'index_equals'}\n";
	print "<option value=1>$text{'index_contains'}\n";
	print "<option value=2>$text{'index_nequals'}\n";
	print "<option value=3>$text{'index_ncontains'}\n";
	print "<option value=6>$text{'index_lower'}\n";
	print "<option value=7>$text{'index_higher'}\n";
	print "</select> <input name=what size=15>&nbsp;&nbsp;\n";
	print "<input type=submit value=\"$text{'find'}\"></form>\n";
	}
elsif (@glist) {
	# Show table of all groups
	print "<hr>\n";
	print "<a href='edit_group.cgi?new=1'>$text{'index_gadd'}</a><br>\n";
	@glist = &useradmin::sort_groups(\@glist, $mconfig{'sort_mode'});
	&useradmin::groups_table(\@glist, $form++, 0);
	}
elsif ($access{'gcreate'} || !@allglist) {
	# Show none message
	print "<hr>\n";
	$base = &get_user_base();
	print "<b>",&text('index_gnone', "<tt>$base</tt>"),"</b><p>\n";
	}
if ($access{'gcreate'}) {
	print "<a href='edit_group.cgi?new=1'>$text{'index_gadd'}</a><p>\n";
	}

&ui_print_footer("/", $text{'index'});

